#include <IRremote.h>
#include <toneAC.h>

const int IR_RECEIVER_PIN = 8;
const int LED_PINS[] = {2, 3, 4, 5, 6, 7};
const int BUZZER_PIN = 9;

IRrecv irrecv(IR_RECEIVER_PIN);
decode_results results;

int tones[] = {262, 294, 330, 349, 392, 440, 494, 523, 587, 659, 698, 784};
int currentToneIndex = 0;

unsigned long lastCommandTime = 0;
unsigned long debounceDelay = 500;  // Set an appropriate debounce delay in milliseconds

void setup() {
  Serial.begin(9600);
  irrecv.enableIRIn();

  for (int i = 0; i < sizeof(LED_PINS) / sizeof(LED_PINS[0]); i++) {
    pinMode(LED_PINS[i], OUTPUT);
  }

  flashLEDs(2);
}

void loop() {
  if (irrecv.decode(&results)) {
    // Check for debouncing
    if (millis() - lastCommandTime > debounceDelay) {
      Serial.print("Hex Value: ");
      Serial.println(results.value, HEX);

      playTone(results.value);

      lastCommandTime = millis();  // Update the last command time
    }

    irrecv.resume();
  }

  // Flash LEDs in sequence and then in reverse
  flashLEDsSequence();
  flashLEDsReverse();
}

void playTone(unsigned long value) {
  // Map the hex values to corresponding tones
  int index = -1;
  switch (value) {
    case 0xFFA25D: index = 0; break; // CH-
    case 0xFF629D: index = 1; break; // CH
    case 0xFFE21D: index = 2; break; // CH+
    case 0xFF22DD: index = 3; break; // <<
    case 0xFF02FD: index = 4; break; // >>
    case 0xFFC23D: index = 5; break; // >||
    case 0xFFE01F: index = 6; break; // -
    case 0xFFA857: index = 7; break; // +
    case 0xFF906F: index = 8; break; // EQ
    case 0xFF9867: index = 9; break; // 100+
    case 0xFFB04F: index = 10; break; // 200+
    case 0xFF6897: index = 11; break; // 0
    case 0xFF30CF: index = 12; break; // 1
    case 0xFF18E7: index = 13; break; // 2
    case 0xFF7A85: index = 14; break; // 3
    case 0xFF10EF: index = 15; break; // 4
    case 0xFF38C7: index = 16; break; // 5
    case 0xFF5AA5: index = 17; break; // 6
    case 0xFF42BD: index = 18; break; // 7
    case 0xFF4AB5: index = 19; break; // 8
    case 0xFF52AD: index = 20; break; // 9
  }

  // Play the corresponding tone
  if (index != -1) {
    currentToneIndex = index; // Update the current tone index
    toneAC(BUZZER_PIN, tones[index], 500);
    delay(500);
    noToneAC(); // Stop the tone before playing the next one
    delay(50); // Small delay before starting the next tone
  }
}

void flashLEDs(int times) {
  for (int i = 0; i < times; i++) {
    for (int j = 0; j < sizeof(LED_PINS) / sizeof(LED_PINS[0]); j++) {
      digitalWrite(LED_PINS[j], HIGH);
      delay(100);
      digitalWrite(LED_PINS[j], LOW);
    }
    delay(500);
  }
}

void flashLEDsSequence() {
  for (int i = 0; i < sizeof(LED_PINS) / sizeof(LED_PINS[0]); i++) {
    digitalWrite(LED_PINS[i], HIGH);
    delay(100);
    digitalWrite(LED_PINS[i], LOW);
  }
}

void flashLEDsReverse() {
  for (int i = sizeof(LED_PINS) / sizeof(LED_PINS[0]) - 1; i >= 0; i--) {
    digitalWrite(LED_PINS[i], HIGH);
    delay(100);
    digitalWrite(LED_PINS[i], LOW);
  }
}
